#!/usr/bin/perl
#
# Jeremy Chartier, <jeremy.chartier@free.fr>
# Date: 2003/03/03
# Revision: 1.8
# 

use Getopt::Long;		# use Getopt for options
use Socket;                     # use socket for resolving domain name from IP

$domains_file = "domains";
$rules_file = "rules";
$addr_len = 15;
$nb_len = 6;
$port_len = 5;
$attack_len = 90;
$class_len = 60;
$resolve_len = 50;
$hour_len = 2;
$graph_len = 75;
$prior_len = 1;
$sever_len = 7;
$domain_len = 30;
$ether_len = 10;

$version = "1.9.rc2";
$datever = "2003/04/09 20:06:53";
$date = `date '+%Y/%m/%d at %H:%M:%S'`;
$dategraph = `date +%Y%m%d%H%M`;
chomp $dategraph;

Getopt::Long;

GetOptions( \%opt,
	'd',
	'r',
	'i',
	'g=s',
	'h=s',
	'n=i',
	'c',
	'ether=s',
	'class',
	'src',
	'dst',
	'class_attack',
	'attack',
	'dport',
	'dport_attack',
	'nids',
	'stateful',
	'interfaces',
	'severity',
	'src_attack',
	'dst_attack',
	'hour',
	'proto',
	'hour_attack',
	'daily_event',
	'domain_src',
	'src_dst_attack',
	'portscan',
	'priority',
	'genref=s'
	) or usage();
	


if ( $opt{genref} ) {
	open(RULES,">$opt{genref}") or die "Can not open domain file: $opt{genref} $!\n";
	while (<>) {
		next if ($_ eq "" or /^#/);
		if ( $_ =~ m/reference/ox ) {
			$_ =~ s/^alert\s(\w+)\s.+\s.+\>\s.+\s.+\(msg\:\"([^\"|^\"]*?)\".+reference\://x;
			$PROTO = $1; $PROTO =~ tr/a-z/A-Z/; $SIG = $2;
			$_ = ";$_";
			$_ =~ s/^\;([^\;|^\;]*?)\;.*$//x;
			$REF = $1;
			print RULES ("$SIG {$PROTO}\t\t$REF\n");
		}
	}
	close (RULES);
} else {	
# process whatever comes in
while (<>) {
my $alert = {};
chomp;
# if the line is blank, go to the next one
next if $_ eq "";
#
#  test if the log correspond to the first line of snort's full alert
#
if ( $_ =~ /^\[\*\*\]/ ) {
	if ( $_ =~ m/^\[\*\*\]\s+\[[0-9:]+\]\s\(spp_portscan\d?\)\sPortscan\sdetected\sfrom\s([\d\.]+)/ox) { # portscan
		$alert->{SADDR} = $2; $alert->{TYPE} = "plugin"; $alert->{ETHER} = $1;
		$alert->{PLUGIN} = "portscan";
		process_data($alert); next;
	} else {
		s/\s*\[\*\*\]\s*//og;
		s/\s*\[[0-9:]+\]\s*//o;
		#
		#  verify if there are an interface in the log line
		#  anf if it's the case, add interface in databse
		#  
		s/\<([^\<|^\>]*?)\>//og;  # Get interface
		$ETHER = $1;
		if ( ( $opt{ether} ) && ( $opt{ether} ne "$1" ) ) {
			next; next; next;
		}
		$SIG = $_;
		next;
	}
}


#
#  test if the log correspond to the second line of snort's full alert
#
if ( $_ =~ m/^\[Classification\:([^\[|^\]]*?)\]\s+\[Priority\:\s+(\d{1})\]/ox ) {
	if ($2 == 1) {
		$severity = "high";
	} elsif ($2 == 2) {
		$severity = "medium";
	} elsif ($2 == 3) {
		$severity = "low";
	} else {
		$severity = "unknown"
	}
	$CLASS = $1;
	$PRIORITY = $2;
	$SEVERITY = $severity;
	next;
}



#
#  test if the log correspond to the third line of snort's full alert
#
if ( $_ =~ /^(\d{2})\/(\d{2})\-(\d{2})\:(\d{2})\:(\d{2})\.\d+\s+([\d\.]+)\:?([\d]*)\s[\-\>]+\s([\d\.]+)\:?([\d]*)/) {
	$MON = $1; $DAY = $2; $HOUR = $3; $MIN = $4; $SEC = $5; $SADDR = $6; $SPORT = $7; $DADDR = $8; $DPORT = $9;
	next;
}




#
#  test if the log correspond to the fourth line of snort's full alert
#
if ( $_ =~ m/^(\w+)\s.+ID:\d+\sIpLen\:\d+\sDgmLen\:\d+/ox ) {
	$alert->{PROTO} = $1; 
	#
	#  test if option <ether> is selected and if the option's
	#  parameter correspond to the interface line's log
	#  else pass the log
	#  
	if ( $opt{ether} ) {
		if ( $opt{ether} eq "$ETHER" ) {
			$alert->{MON} = $MON;   $alert->{DAY} = $DAY;   $alert->{HOUR} = $HOUR;
			$alert->{MIN} = $MIN;   $alert->{SEC} = $SEC;   $alert->{SADDR} = $SADDR;
			$alert->{SPORT} = $SPORT;   $alert->{DADDR} = $DADDR;   $alert->{DPORT} = $DPORT;
			$alert->{HOST} = "localhost";
			$alert->{SIG} = $SIG;
			$alert->{TYPE} = "sys";
			$alert->{PLUGIN} = "none";
			$alert->{CLASS} = $CLASS;
			$alert->{PRIORITY} = $PRIORITY;
			$alert->{SEVERITY} = $SEVERITY;
			$alert->{ETHER} = $ETHER;
			$alert->{PROTO} = $1;
			process_data($alert);
			next;
		}
	} else {
		$alert->{MON} = $MON;   $alert->{DAY} = $DAY;   $alert->{HOUR} = $HOUR;
		$alert->{MIN} = $MIN;   $alert->{SEC} = $SEC;   $alert->{SADDR} = $SADDR;
		$alert->{SPORT} = $SPORT;   $alert->{DADDR} = $DADDR;   $alert->{DPORT} = $DPORT;
		$alert->{HOST} = "localhost";
		$alert->{SIG} = $SIG;
		$alert->{TYPE} = "sys";
		$alert->{PLUGIN} = "none";
		$alert->{CLASS} = $CLASS;
		$alert->{PRIORITY} = $PRIORITY;
		$alert->{SEVERITY} = $SEVERITY;
		$alert->{ETHER} = $ETHER;
		$alert->{PROTO} = $1;
		process_data($alert);
		next;
	}
}





#
#  test if the log correspond to a fast alert
#
if ( $_ =~ /\d{2}\/\d{2}\-\d{2}\:\d{2}\:\d{2}\.\d+\s+\[\*\*\]/ ) {
#	$line = <>;
#	chomp($line);
	unless ( $_ eq "" ) {
		# strip off the [**] from either end.
		s/\s*\[\*\*\]\s*//og;
		s/\s*\[[0-9:]+\]\s*//o;
		if ( $_ =~ m/^(\d+)\/(\d+)\-(\d+)\:(\d+)\:(\d+)\.(\d+)(.*)/ox ) {
#		   ([\d\.]+)[\:]*([\d]*)\s[\-\>]+\s([\d\.]+)[\:]*([\d]*)\s(.+)/ox) {
			$alert->{MON} = $1;   $alert->{DAY} = $2;   $alert->{HOUR} = $3; 
			$alert->{MIN} = $4;   $alert->{SEC} = $5;   $alert->{SIG} = $7; 
			$alert->{HOST} = "localhost";
			$alert->{SIG} =~ s/\s*\[[\d\:]+\]\s*//;  # Get rid of [343:33:31]
			$alert->{SIG} =~ s/\[\*\*\]//og; # Get rid of [**] if fast alert
			#
			#  verify if there are an interface in the log line
			#  anf if it's the case, add interface in databse
			#  else pass the log
			#  
			$alert->{SIG} =~ s/\<([^\<|^\>]*?)\>//og;  # Get interface
			$alert->{ETHER} = $1;
			#
			#  test if option <ether> is selected and if the option's
			#  parameter correspond to the interface line's log
			#  
			if ( ( $opt{ether} ) && ( $opt{ether} ne "$1" ) ) {
				next;
			}
			if ($alert->{SIG} =~ m/^\(spp_portscan\d?\)\sPortscan\sdetected\sfrom\s([\d\.]+)/ox) { # portscan
				$alert->{SADDR} = $1; $alert->{TYPE} = "plugin"; 
				$alert->{PLUGIN} = "portscan";
				process_data($alert); $lastwassnort = 1; next;
			} elsif ( $alert->{SIG} =~ s/\s([\d\.]+)[\:]?([\d]*)\s[\-\>]+\s
                           ([\d\.]+)[\:]?([\d]*)\s*//x) {
				$alert->{SADDR} = $1; $alert->{SPORT} = $2; 
				$alert->{DADDR} = $3; $alert->{DPORT} = $4; 
				if ($alert->{SIG} =~ m/spp_anomsensor\:\sAnomaly\sthreshold\s
					  exceeded\:\s([\d\.]+)/ox) { # spade
					$alert->{THR} = $1; $alert->{TYPE} = "plugin"; 
					$alert->{PLUGIN} = "anomsensor"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ s/spp_bo\:\s//ox) { # bo
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "bo"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ /spp_stream4/) { # stream4 
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "stream4"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ /spp_frag2/) { # frag2 
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "frag2"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} elsif ($alert->{SIG} =~ /snort_decoder/) { # snort_decoder 
					$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "snort_decoder"; $alert->{SEVERITY} = "unknown";
					process_data($alert); $lastwassnort = 1; next;
				} else { # normal alert
					if ( $alert->{SIG} =~ s/\[Classification\:([^\[|^\]]*?)\]\s*(?:\[Priority\:\s(\d)\])\s\{([^\{|^\}]*?)\}//x ) {
						if ($2 == 1) {
							$severity = "high";
						} elsif ($2 == 2) {
							$severity = "medium";
						} elsif ($2 == 3) {
							$severity = "low";
						} else {
							$severity = "unknown"
						}
						$alert->{CLASS} = $1; $alert->{PRIORITY} = $2; $alert->{SEVERITY} = $severity; $alert->{PROTO} = $3; 
					}
					$alert->{TYPE} = "sys"; $alert->{PLUGIN} = "none";
					process_data($alert); $lastwassnort = 1; next;
				}
			} else { 
				print STDERR "No source/dest IP address found! Skipped! --> $_\n" if $opt{d}; 
				next;
    			}
		}
	} 
}


#
#  test if the log correspond to a syslog alert
#
if ($_ =~ m/^(\w{3})\s+(\d+)\s(\d+)\:(\d+)\:(\d+)\s 
                   (\S+?)\ssnort[\[\d+\]]*\:\s+(.+)/ox 
      || m/^(\d+)\/(\d+)\-(\d+)\:(\d+)\:(\d+)\.(\d+)\s(.+)/ox) 
{
	$alert->{MON} = $1;    $alert->{DAY} = $2;    $alert->{HOUR} = $3; 
	$alert->{MIN} = $4;    $alert->{SEC} = $5;    $alert->{HOST} = $6;  
	$alert->{SIG} = $7;    
	$alert->{SIG} =~ s/\s*\[[\d\:]+\]\s*//;  # Get rid of [343:33:31]
	$alert->{SIG} =~ s/\[\*\*\]//og; # Get rid of [**] if fast alert
	if ($alert->{SIG} =~ m/^\(spp_portscan\d?\)\sPortscan\sdetected\sfrom\s([\d\.]+)/ox || m/spp_portscan:\sPORTSCAN\sDETECTED\sfrom\s([\d\.]+)/ox) { # portscan
		$alert->{SADDR} = $1; $alert->{TYPE} = "plugin"; 
		$alert->{PLUGIN} = "portscan";
		process_data($alert); $lastwassnort = 1; next;
	} elsif ( $alert->{SIG} =~ s/\s([\d\.]+)[\:]?([\d]*)\s[\-\>]+\s
                           ([\d\.]+)[\:]?([\d]*)\s*//x) {
		$alert->{SADDR} = $1; $alert->{SPORT} = $2; 
		$alert->{DADDR} = $3; $alert->{DPORT} = $4; 
		if ($alert->{SIG} =~ m/spp_anomsensor\:\sAnomaly\sthreshold\s
			  exceeded\:\s([\d\.]+)/ox) { # spade
			$alert->{THR} = $1; $alert->{TYPE} = "plugin"; 
			$alert->{PLUGIN} = "anomsensor"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ s/spp_bo\:\s//ox) { # bo
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "bo"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /spp_stream4/) { # stream4 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "stream4"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /spp_frag2/) { # frag2 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "frag2"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /spp_portscan/) { # frag2 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "portscan"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} elsif ($alert->{SIG} =~ /snort_decoder/) { # snort_decoder 
			$alert->{TYPE} = "plugin"; $alert->{PLUGIN} = "snort_decoder"; $alert->{SEVERITY} = "unknown";
			process_data($alert); $lastwassnort = 1; next;
		} else { # normal alert
			if ( $alert->{SIG} =~ s/\[Classification\:([^\[|^\]]*?)\]\s*(?:\[Priority\:\s(\d)\]\:\s)//x ) {
				if ($2 == 1) {
					$severity = "high";
				} elsif ($2 == 2) {
					$severity = "medium";
				} elsif ($2 == 3) {
					$severity = "low";
				} else {
					$severity = "unknown"
				}
				$alert->{CLASS} = $1; $alert->{PRIORITY} = $2; $alert->{SEVERITY} = $severity;  
			}
			$alert->{TYPE} = "sys"; $alert->{PLUGIN} = "none";
			#
			#  verify if there are an interface in the log line
			#  anf if it's the case, add interface in databse
			#  
			if ( $alert->{SIG} =~ m/\</ ) {
				$alert->{SIG} =~ s/\<([^\<|^\>]*?)\>\s+//og;  # Get interface
				$alert->{ETHER} = $1;
				#
				#  test if option <ether> is selected and if the option's
				#  parameter correspond to the interface line's log
				#  else pass the log
				#  
				if ( ( $opt{ether} ) && ( $opt{ether} ne "$1" ) ) {
					next;
				}
			}
			$alert->{SIG} =~ m/\{([^\{|^\}]*?)\}/;  # Get interface
			$alert->{PROTO} = $1;
			process_data($alert); $lastwassnort = 1; next;
		}
	} else {
		print STDERR "No source/dest IP address found! Skipped! --> $_\n" if $opt{d}; 
		$alert = {}; next;
	}
}
# If a snort message has been repeated several times
elsif ($lastwassnort && $_ =~ m/last message repeated (\d+) times/) {
# put the data in the matrix again for each repeat
	$repeats = $1;
	while ($repeats) {
		push @result, $result[-1];
		$repeats--;
	}
	next;
} else {
	$lastwassnort = 0;
	next;
}				# Message not related to snort
}

# begin statistics
# I should've used $#result + 1 as $total in the first version! :(
$total = $#result + 1;
$totalscan = $#portscan + 1;
$opt_n = 1 if ( $opt{n} );
$opt_h = 1 if ( $opt{h} );
$opt_g = 1 if ( $opt{g} );
$val_opt_n = $opt{n} if ( $opt_n == 1 );
$val_opt_h = $opt{h} if ( $opt_h == 1 );
$val_opt_g = $opt{g} if ( $opt_g == 1 );

if ($opt{n}) {
	$nb=$opt{n};
} else {	
	$nb=10000;
}

if ( $total == 0 ) {
	print "No Snort's logs found\n";
	exit
} else {
$report=$opt{report} if $opt{report};

for $i ( 0 .. $#result ) {
  # for the same pair of attacker and victim with same attack
  # used in same_src_dst_attack()
  $s0{"$result[$i]->[9],$result[$i]->[7],$result[$i]->[6]"}++;
  # used in same_src_dst_attack()
  $s1{"$result[$i]->[15]"}++;
  # used in same_src_attack()
  $s2{"$result[$i]->[6],$result[$i]->[7],$result[$i]->[14]"}++;
  # to same victim with same method
  # to see how many attacks received by one host
  # used in same_dest_attack()
  $s3{"$result[$i]->[6],$result[$i]->[9],$result[$i]->[14]"}++;
  # same signature
  # to see the popularity of one attack method
  # used in attack()
  $s4{"$result[$i]->[6],$result[$i]->[13],$result[$i]->[14]"}++;
  # used in attack_src()
  $s5{"$result[$i]->[7]"}++;
  # used by attack_dst 
  $s6{"$result[$i]->[9]"}++;
  # used by hour()
  $s7{"$result[$i]->[2]"}++;
  # used by hour()
  $s8{"$result[$i]->[2],$result[$i]->[14]"}++;
  # use by same_hour_attack()
  $s9{"$result[$i]->[2],$result[$i]->[6]"}++;
  # to see classification
  # used by same_class() 
  $s10{"$result[$i]->[11],$result[$i]->[14]"}++;
  # use by proto()
  $s11{"$result[$i]->[16]"}++;
  # use by nids()
  $s13{"$result[$i]->[5]"}++;
  # use by daily_event()
  $s15{"$result[$i]->[1],$result[$i]->[0],$result[$i]->[14]"}++;
  # use by dport()
  $s16{"$result[$i]->[10]"}++;
  # use by stateful()
  $s17{"$result[$i]->[12]"}++;
  # use by dport_attack()
  $s18{"$result[$i]->[10],$result[$i]->[6]"}++;
  # use by daily_event()
  $s19{"$result[$i]->[1],$result[$i]->[0]"}++;
  # use by severity()
  $s20{"$result[$i]->[14]"}++;
}
 

$processor = keys(%s17);
$anomaly = --$processor;


init_domains();
init_rules();
print_head();

if ( $opt{src} ) {
	attack_src();
} elsif ( $opt{dst} ) {
	attack_dst();
} elsif ( $opt{src_attack} ) {
	same_src_attack();
} elsif ( $opt{dst_attack} ) {
	same_dst_attack();
} elsif ( $opt{src_dst_attack} ) {
	same_src_dst_attack();
} elsif ( $opt{attack} ) {
	attack();
} elsif ( $opt{class} ) {
	same_class();
} elsif ( $opt{severity} ) {
	severity();
} elsif ( $opt{daily_event} ) {
	daily_event() if ($result[0]->[0] ne $result[$#result]->[0]) || ($result[0]->[1] < $result[$#result]->[1]);
} elsif ( $opt{hour} ) {
	hour();
} elsif ( $opt{hour_attack} ) {
	same_hour_attack();
} elsif ( $opt{dport} ) {
	dport();
} elsif ( $opt{dport_attack} ) {
	dport_attack();
} elsif ( $opt{nids} ) {
	nids() if (keys(%s13) > 1);
} elsif ( $opt{stateful} ) {
	stateful() if ($anomaly > 0);
} elsif ( $opt{domain_src} ) {
	domain_src();
} elsif ( $opt{proto} ) {
	proto();
} elsif ( $opt{portscan} ) {
	portscan() if ( $totalscan > 0 );
} elsif ( $opt{interfaces} ) {
	interfaces() if ( keys(%s1) > 1 );
} else {
# begin report
#print_head();
print_summary();
print_menu() if $opt{h};

proto();
severity();
hour();
daily_event() if ($result[0]->[0] ne $result[$#result]->[0]) || ($result[0]->[1] < $result[$#result]->[1]);
dport();
nids() if (keys(%s13) > 1);
interfaces() if (keys(%s1) > 1);
domain_src() if ( $opt{c} );

attack_src();
same_src_attack();
attack_dst();
same_dst_attack();
same_src_dst_attack();
attack();
same_class();
same_hour_attack();
dport_attack();
stateful() if ($anomaly > 0);
portscan() if ( $totalscan > 0 );
print_footer();
}
}
}

# print the header (e.g. for mail)
sub print_head {
if ( $opt_h == 1 ) {
	open (FILEOUT, ">$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<HTML>\n<HEAD>\n<STYLE>\n<!--\n");
	print FILEOUT ("BODY { font-family: serif; background-color: #FFEFC6; font-size: 12px; }\n");
	print FILEOUT ("/* Table Header */\n");
	print FILEOUT (".TH { font-family: Verdana,Arial,Helvetica,sans-serif; font-size: 12px; background-color: #FFEFC6; color: blue; font-weight: bold; }\n");
	print FILEOUT ("/* Table body */\n");
	print FILEOUT (".TB { color: black; background-color: white; font-size: 12px; }\n");
	print FILEOUT ("/* Table body RED*/\n");
	print FILEOUT (".TBR { color: red; background-color: white; font-size: 12px; }\n");
	print FILEOUT ("/* Table body GREEN*/\n");
	print FILEOUT (".TBG { color: green; background-color: white; font-size: 12px; }\n");
	print FILEOUT ("-->\n</STYLE>\n</HEAD>\n");
	print FILEOUT ("<BODY>\n");
	print FILEOUT ("<h1>IDS Statistics generated the $date</h1>\n");
	close (FILEOUT);
} else {
	print "subject: ids report generated the $date\n"; 
}
}

# print the time of begin and end of the log
sub print_summary {
	if ( $opt_h ==1 ) {
		open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
		print FILEOUT ("<TABLE>\n");
		print FILEOUT ("<UL><A NAME=\"top\"></A>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>The log begins at:</TH>\n");
		print FILEOUT ("<TD>$result[0]->[0] $result[0]->[1] $result[0]->[2]:$result[0]->[3]:$result[0]->[4]</TD><TD WIDTH=170></TD><TD><B>Legende:</B></TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>The log ends at:</TH>\n");
		print FILEOUT ("<TD>$result[$#result]->[0] $result[$#result]->[1] $result[$#result]->[2]:$result[$#result]->[3]:$result[$#result]->[4]</TD><TD></TD><TD><FONT COLOR= RED>RED : Dangerous</FONT></TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Total events:</TH><TD> $total</TD><TD WIDTH=50></TD><TD><FONT COLOR= GREEN>GREEN : Warning</FONT></TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>NIDS recorded:</TH><TD> ". keys(%s13) ." with ". keys(%s1) . " interface(s)   </TD><TD></TD><TD><FONT COLOR= BLACK>BLACK : Normal</FONT></TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Signatures recorded:</TH><TD> ". keys(%s4) ."</TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Classification recorded:</TH><TD> ". keys(%s10) ."</TD><TD></TD><TD><B>Domains File:</B> $domains_file</TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Source IP recorded:</TH><TD> ". keys(%s5) ."</TD><TD></TD><TD><B>Number of domains:</B> ". keys(%DomainName) ."</TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Destination IP recorded:</TH><TD> ". keys(%s6) ."</TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Preprocessor anomaly:</TH><TD> ". $anomaly ."</TD><TD></TD><TD><B>Rules File:</B> $rules_file</TD></TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Severity recorded:</TH><TD> ". keys(%s20) ."</TD><TD></TD><TD><B>Number of referenced rules:</B> ". keys(%link) ."</TR>\n");
		print FILEOUT ("<TR ALIGN=LEFT><TH>Portscan detected:</TH><TD> ", eval '$#posres +1',"</TD></TR>\n") if $opt_p;
		print FILEOUT ("<TR ALIGN=LEFT><TH>Anomaly detected:</TH><TD> ", eval '$#anores +1',"</TD></TR>\n") if $opt_n;
		print FILEOUT ("</TABLE>\n");
		print FILEOUT ("<HR>\n");
		close (FILEOUT);

	} else {
		print "The log begins from: $result[0]->[0] $result[0]->[1] $result[0]->[2]:$result[0]->[3]:$result[0]->[4]\n";
		print "The log ends     at: $result[$#result]->[0] $result[$#result]->[1] $result[$#result]->[2]:$result[$#result]->[3]:$result[$#result]->[4]\n";
		print "Total events: $total\n";
		print "NIDS recorded: ". keys(%s13) ." with ". keys(%s1) ." interface(s)\n";
		print "Signatures recorded: ". keys(%s4) ."\n";
		print "Classification recorded: ". keys(%s10) ."\n";
		print "Source IP recorded: ". keys(%s5) ."\n";
		print "Destination IP recorded: ". keys(%s6) ."\n";
		print "Preprocessor anomaly: ". $anomaly ."\n";
		print "Severity recorded: ". keys(%s20) ."\n";
		print "Portscan recorded: ", eval '$#posres +1',"\n" if $opt_p;
		print "Anomaly recorded: ", eval '$#anores +1',"\n" if $opt_n;
	}
}

# print menu for HTML page
sub print_menu {
	if ( $opt_h == 1) {
		open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
		print FILEOUT ("<TABLE BORDER=0><TR ALIGN=left><TD WIDTH=50% VALIGN=top>\n");
		print FILEOUT ("<U><B>General Statistics</B></U><P>\n");
		print FILEOUT ("<li><a href=\"#proto\">The distribution of event by protocoles</a>\n");
		print FILEOUT ("<li><a href=\"#severity\">The distribution of event by severity</a>\n");
		print FILEOUT ("<li><a href=\"#hour\">The distribution of event by hour</a>\n");
		print FILEOUT ("<li><a href=\"#daily_event\">The distribution of event by day</a>\n") if ($result[0]->[0] ne $result[$#result]->[0]) || ($result[0]->[1] < $result[$#result]->[1]);
		print FILEOUT ("<li><a href=\"#interfaces\">The distribution of event by interfaces</a>\n") if (keys(%s1) > 1);
		print FILEOUT ("<li><a href=\"#dport\">The distribution of event by destination port</a>\n");
		print FILEOUT ("<li><a href=\"#nids\">The distribution of event by sonde</a>\n") if (keys(%s13) > 1);
		print FILEOUT ("<li><a href=\"#domain_src\">To see the popularity of one domain</a>\n") if ( $opt{c} );
	
		print FILEOUT ("</TD><TD WIDTH=50% VALIGN=top>\n");
		
		print FILEOUT ("<U><B>Specific Statistics</B></U><P>\n");
		print FILEOUT ("<li><a href=\"#attack_src\">Popularity of one source host</a>\n");
		print FILEOUT ("<li><a href=\"#same_src_attack\">Events from one host to any with same method</a>\n");
		print FILEOUT ("<li><a href=\"#attack_dst\">Popularity of one destination host</a>\n");
		print FILEOUT ("<li><a href=\"#same_dst_attack\">Events to one host from any with same method</a>\n");
		print FILEOUT ("<li><a href=\"#same_src_dst_attack\">Events from a host to a destination</a>\n");
		print FILEOUT ("<li><a href=\"#port_attack\">Events to one destination port grouped by attack</a>\n");
		print FILEOUT ("<li><a href=\"#attack\">Distribution of attack methods</a>\n");
		print FILEOUT ("<li><a href=\"#same_class\">Ddistribution of classification method</a>\n");
		print FILEOUT ("<li><a href=\"#stateful\">Distribution of stateful problems</a>\n") if ($anomaly > 0);
		print FILEOUT ("<li><a href=\"#hour_attack\">Events by hour</a>\n");
		print FILEOUT ("<li><a href=\"#portscan\">Portscan list</a>\n") if ( $totalscan > 0 );
		print FILEOUT ("</ul>\n");
		print FILEOUT ("</TD></TR></TABLE><HR><BR>\n");
		close (FILEOUT);
	}
}       







sub daily_event {
$max = 0;
if ( $opt_h == 1 ) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"daily_event\" HREF=\"#top\">The distribution of event by day</a></h3>\n");
	print FILEOUT ("<table border=5 align=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>Jour</td><td>Mois</td><td>Nb</td><td>%</td><td>Graph</td></tr>");

	foreach $k (%s19) {
		$max = $s19{$k} if ($s19{$k} > $max);
	}
	my $ratio = $max/$total*100;
	foreach $k (sort { $a <=> $b } keys %s19) {
		@_ = split ",",$k;
		my $num = $s19{$k}/$total*(50000/$ratio);
		printf FILEOUT ("<tr class=TB><td>%-${hour_len}d</td><td>%-3s</td><td>%-${nb_len}d</td><td>%-2.2f</td><td><hr noshade align=left size=10 color=#CCCCCC  width=%3d ></td></tr>\n",$_[0],$_[1],$s19{$k},$s19{$k}/$total*100,$num);
		push @list1 ,($s19{$k});
		push @list2 ,("$_[0]");
	}
	print FILEOUT ("</TD></TR></TABLE><BR>\n");
	init_graph("days1","area") if $opt{g};

	$compt = 1;	
	foreach $k (sort { $a <=> $b } keys %s15) {
		@_ = split ",",$k;
		$beforeday = $_[0];
		$belowday = $beforeday if ( $compt == 1 );
		if ( $belowday < $beforeday ) {
			if ( $low ) {
				push @listlow ,($low);
				undef $low;
			} else {
				push @listlow ,(0);
			}
			if ( $medium ) {
				push @listmedium ,($medium);
				undef $medium;
			} else {
				push @listmedium ,(0);
			}
			if ( $high ) {
				push @listhigh ,($high);
				undef $high;
			} else {
				push @listhigh ,(0);
			}
			if ( $unknown ) {
				push @listunknown ,($unknown);
				undef $unknown;
			} else {
				push @listunknown ,(0);
			}
		}
		if ( $_[2] eq "low") {
			$low = $s15{$k};
		} elsif ( $_[2] eq "medium" ) {
			$medium = $s15{$k};
		} elsif ( $_[2] eq "high" ) {
			$high = $s15{$k};
		} elsif ( $_[2] eq "unknown" ) {
			$unknown = $s15{$k};
		}
		$belowday = $beforeday;
		$compt ++;
	}
	if ( $low ) {
		push @listlow ,($low);
		undef $low;
	} else {
		push @listlow ,(0);
	}
	if ( $medium ) {
		push @listmedium ,($medium);
		undef $medium;
	} else {
		push @listmedium ,(0);
	}
	if ( $high ) {
		push @listhigh ,($high);
	} else {
		push @listhigh ,(0);
	}
	if ( $unknown ) {
		push @listunknown ,($unknown);
		undef $unknown;
	} else {
		push @listunknown ,(0);
	}
	if ( $opt{g} ) {
		init_graph("days2","lines");
		print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><IMG SRC=".$val_opt_h._days1_.$dategraph.".".$opt{g}."</TD></TR><BR><TR><TD><IMG SRC=".$val_opt_h._days2_.$dategraph.".".$opt{g}."></TD></TR></TABLE>\n");
		print FILEOUT ("</TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	foreach $k (%s19) {
		$max = $s19{$k} if ($s19{$k} > $max);
	}
	$ratio = $max/$total*100;
	section_header("The distribution of event by day\n", "daily_event");
	foreach $k (sort { $a <=> $b } keys %s19) {
		@_ = split ",",$k;
		$num = $s19{$k}/$total*75*(100 / $ratio);
		printf("  %-${hour_len}d   %-3s   %-${nb_len}d  %5.2f  %-${graph_len}s\n", $_[0],$_[1],$s19{$k},$s19{$k}/$total*100, '#' x $num);
	}
}
}


sub severity {
$max = 0;
if ( $opt_h == 1 ) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<H3><CENTER><A NAME=\"severity\" HREF=\"#top\">The distribution of severity</A></CENTER></H3>\n");
	#print "<table border=5>\n";
	print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><TABLE BORDER=5 ALIGN=center>\n");
	print FILEOUT ("<TR CLASS=TH ALIGN=center><TD>%</TD><TD>Nb</TD><TD>Severity</TD></TR>");
	foreach $k (sort { $s20{$b} <=> $s20{$a} } keys %s20) {
		$STYLE = init_color($k);
		printf FILEOUT (("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${sever_len}s</td></tr>\n", $s20{$k}/$total*100,$s20{$k},$k));
		push @list1 ,($s20{$k}/$total*100);
		push @list2 ,("$k");
	}
	if ( $opt{g} ) {
		init_graph("severity","pie");
		print FILEOUT ("</TABLE></TD><TD WIDTH=300 ALIGN=right><IMG SRC=".$val_opt_h._severity_.$dategraph.".".$opt{g}."></TD></TR></TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE></TD></TR></TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	foreach $k (%s20) {
		$max = $s20{$k} if ($s20{$k} > $max);
	}
	$ratio = $max/$total*100;
	section_header("The distribution of severity\n", "severity");
	foreach $k (sort { $s20{$b} <=> $s20{$a} } keys %s20) {
		$num = $s20{$k}/$total*75*(100 / $ratio);
		printf("  %-5.2f  %-${nb_len}d  %-${sever_len}s  %-${graph_len}s\n", $s20{$k}/$total*100,$s20{$k},$k, '#' x $num);
	}
}
}



sub interfaces {
if ($opt{h}) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<H3><CENTER><A NAME=\"interfaces\" HREF=\"#top\">The distribution of event by interfaces</A></CENTER></H3>\n");
	print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><TABLE BORDER=5 ALIGN=center>\n");
	print FILEOUT ("<TR CLASS=TH ALIGN=center><TD>%</TD><TD>Nb</TD><TD>Interface</TD></TR>");
	foreach $k (sort { $s1{$b} <=> $s1{$a} } keys %s1) {
		printf FILEOUT (("<TR CLASS=TB><TD>%2.2f</TD><TD>%-${nb_len}d</TD><TD>%-32s</TD></TR>\n", $s1{$k}/$total*100,$s1{$k},$k));
		push @list1 ,($s1{$k}/$total*100);
		push @list2 ,("$k");
	}
	if ( $opt{g} ) {
		init_graph("ether","pie");
		print FILEOUT ("</TABLE></TD><TD WIDTH=300 ALIGN=right><IMG SRC=".$val_opt_h._ether_.$dategraph.".".$opt{g}."></TD></TR></TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE></TD></TR></TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	section_header("The distribution of event by interfaces\n", "interfaces");
	foreach $k (sort { $s1{$b} <=> $s1{$a} } keys %s1) {
		printf("  %5.2f  %-${nb_len}d  %-10s\n", $s1{$k}/$total*100,$s1{$k},$k); 
	}
}
}





sub init_graph {
use GD::Graph::pie;
use GD::Graph::bars;
use GD::Graph::lines;
use GD::Graph::area;
	my $func = shift;
	my $graph = shift;
	if ( $graph eq "pie" ) {
		$graphiq = GD::Graph::pie->new(250, 250);
		$graphiq->set( 	title => 'Graph') or die $graphiq->error;
		$gd = $graphiq->plot([\@list2, \@list1]) or die $graphiq->error;
	} elsif ( $graph eq "bars" ) {
		$graphiq = GD::Graph::bars->new(700, 300);
		if ( $func eq "hours2" ) {
			$graphiq->set( 	title => 'Number of Events by Hours',
				x_label => "Hours",
				y_label => "Events",
				dclrs => ['blue','green','red','cyan'],
				bar_spacing => 1,
				bar_width => 3,
				long_ticks => 1) or die $graphiq->error;
				$graphiq->set_legend('Low events','Medium events','High event','Unknown events');
				$gd = $graphiq->plot([\@list2, \@listlow, \@listmedium, \@listhigh, \@listunknown]) or die $graphiq->error;
		}
	} elsif ($graph eq "lines" ) {
		$graphiq = GD::Graph::lines->new(700, 300);
		if ( $func eq "days2" ) {
			$graphiq->set( 	title => 'Number of Severity Events by Days',
				x_label => "Days",
				y_label => "Events",
				line_types => [1, 1, 1, 1],
				dclrs => ['blue','green','red','cyan'],
				) or die $graphiq->error;
			$graphiq->set_legend('Low events','Medium events','High event','Unknown events');
			$gd = $graphiq->plot([\@list2, \@listlow, \@listmedium, \@listhigh, \@listunknown]) or die $graphiq->error;
		}
	} elsif ($graph eq "area" ) {
		$graphiq = GD::Graph::area->new(700, 300);
		if ( $func eq "days1" ) {
			$graphiq->set( 	title => 'Number of Events by Days',
				x_label => "Days",
				y_label => "Events",
				) or die $graphiq->error;
			$gd = $graphiq->plot([\@list2, \@list1]) or die $graphiq->error;
		}
		if ( $func eq "hours1" ) {
			$graphiq->set( 	title => 'Number of Events by Hours',
				x_label => "Hours",
				y_label => "Events") or die $graphiq->error;
				$gd = $graphiq->plot([\@list2, \@list1]) or die $graphiq->error;
		}
	}

	open(IMG, '>'.$val_opt_h._.$func._.$dategraph.'.'.$opt{g}) or die $!;
	binmode IMG;
	$opton = $opt{g};
	print IMG $gd->$opton;
	close (IMG);
	# undef @list1;
	# undef @list2;
	undef @listlow;
	undef @listmedium;
	undef @listhigh;
	undef @listunknown;
}



sub nids {
if ($opt{h}) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"nids\" HREF=\"#top\">The distribution of event by sonde</a></CENTER></h3>\n");
	print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Sonde</td></tr>");
	foreach $k (sort { $s13{$b} <=> $s13{$a} } keys %s13) {
		printf FILEOUT (("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td>%-32s</td></tr>\n", $s13{$k}/$total*100,$s13{$k},$k));
		push @list1 ,($s13{$k});
		push @list2 ,("$k");
	}
	if ( $opt{g} ) {
		init_graph("nids","pie");
		print FILEOUT ("</TABLE></TD><TD WIDTH=300 ALIGN=right><IMG SRC=".$val_opt_h._nids_.$dategraph.".".$opt{g}."></TD></TR></TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE></TD></TR></TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	section_header("The distribution of event by sonde\n", "nids");
	foreach $k (sort { $s13{$b} <=> $s13{$a} } keys %s13) {
		printf("  %5.2f  %-${nb_len}d  %-32s\n", $s13{$k}/$total*100,$s13{$k},$k); 
	}
}
}


sub proto {
if ($opt{h}) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"proto\" HREF=\"#top\">The distribution of event by protocoles</a></CENTER></h3>\n");
	print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Protocoles</td></tr>");
	foreach $k (sort { $s11{$b} <=> $s11{$a} } keys %s11) {
		printf FILEOUT (("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td>%-32s</td></tr>\n", $s11{$k}/$total*100,$s11{$k},$k));
		push @list1 ,($s11{$k});
		push @list2 ,("$k");
	}
	if ( $opt{g} ) {
		init_graph("proto","pie");
		print FILEOUT ("</TABLE></TD><TD WIDTH=300 ALIGN=right><IMG SRC=".$val_opt_h._proto_.$dategraph.".".$opt{g}."></TD></TR></TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE></TD></TR></TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	section_header("The distribution of event by protocoles\n", "proto");
	foreach $k (sort { $s11{$b} <=> $s11{$a} } keys %s11) {
		printf("  %5.2f  %-${nb_len}d  %-32s\n", $s11{$k}/$total*100,$s11{$k},$k); 
	}
}
}


sub nids_attack {
$i=0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s14) ) ) {
	$val_opt_n = keys %s14;
}
   section_header("The distribution of event by sonde\n", "nids_attack");
   foreach $k (sort { $s14{$b} <=> $s14{$a} } keys %s14) {
      @_ = split ",",$k;
      printf("  %-5.2f  %-${nb_len}d  %-31s  %-${attack_len}s\n",
	     $s14{$k}/$total*100, $s14{$k},$_[0],$_[1]) if $i < $val_opt_n;
	     ++$i;
    }
}


sub stateful {
if ($opt{h}) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"stateful\" HREF=\"#top\">The distribution of stateful problems</a><CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Processor</td></tr>");
	foreach $k (sort { $s17{$b} <=> $s17{$a} } keys %s17) {
		printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-32s</td></tr>", $s17{$k}/$total*100,$s17{$k},$k) if $k ne "none";
	}
	print FILEOUT ("</TD></TR></TABLE><BR><HR>\n");
	close (FILEOUT);
} else {
	section_header("The distribution of stateful problems\n", "stateful");
	foreach $k (sort { $s17{$b} <=> $s17{$a} } keys %s17) {
		printf("  %5.2f  %-${nb_len}d  %-32s\n", $s17{$k}/$total*100,$s17{$k},$k) if $k ne "none"; 
	}
}
}


sub same_src_dst_attack {
$i=0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s0) ) ) {
	$val_opt_n = keys %s0;
}
if ($opt{h}) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"same_src_dst_attack\" href=\"#top\">Percentage and number of attacks from a host to a destination</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 align=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>IP Destination</td><td>Attack</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s0{$a} <=> $s0{$b} } keys %s0) { 
			@_ = split ",",$k;
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td></tr>\n", $s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	} else {
		foreach $k (sort { $s0{$b} <=> $s0{$a} } keys %s0) { 
			@_ = split ",",$k;
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td></tr>\n", $s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("The number of attacks from same host to same destination using same method\n", "same_src_dst_attack");
	if ($opt{i}) {
		foreach $k (sort { $s0{$a} <=> $s0{$b} } keys %s0) { 
			@_ = split ",",$k;
			printf("  %-5.2f  %-${nb_len}d  %-${addr_len}s  %-${addr_len}s  %-${attack_len}s\n",
			$s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	} else {
		foreach $k (sort { $s0{$b} <=> $s0{$a} } keys %s0) { 
			@_ = split ",",$k;
			printf("  %-5.2f  %-${nb_len}d  %-${addr_len}s  %-${addr_len}s  %-${attack_len}s\n",
			$s0{$k}/$total*100, $s0{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	}
}
}


# to see how many attacks launched from one host
sub same_src_attack {
$i=0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s2) ) ) {
	$val_opt_n = keys %s2;
}
if ($opt{h}) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"same_src_attack\" href=\"#top\">Percentage and number of attacks from one host to any with same method</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 align=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>Attack</td><td>Severity</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s2{$a} <=> $s2{$b} } keys %s2) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${server_len}s</td></tr>\n", $s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i
		}
	} else {
		foreach $k (sort { $s2{$b} <=> $s2{$a} } keys %s2) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${server_len}s</td></tr>\n", $s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("Percentage and number of attacks from one host to any with same method\n", "same_src_attack");  
	if ($opt{i}) {
		foreach $k (sort { $s2{$a} <=> $s2{$b} } keys %s2) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${server_len}s\n",
			$s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i
		}
	} else {
		foreach $k (sort { $s2{$b} <=> $s2{$a} } keys %s2) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${sever_len}s\n",
			$s2{$k}/$total*100, $s2{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	}
}
}


# to see how many attacks received by one host (destination correlated)
sub same_dst_attack {
$i=0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s3) ) ) {
	$val_opt_n = keys %s3;
}
if ($opt{h}) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"same_dst_attack\" href=\"#top\">Percentage and number of attacks to one host from any with same method</a><CENTER></h3>\n");
	print FILEOUT ("<table border=5 align=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>Attack</td><td>Severity</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s3{$a} <=> $s3{$b} } keys %s3) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${sever_len}s</td></tr>\n",$s3{$k}/$total*100 , $s3{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	} else {
		foreach $k (sort { $s3{$b} <=> $s3{$a} } keys %s3) {
			@_ = split ",",$k;
			$STYLE = init_color($_[2]);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${attack_len}s</td><td>%-${sever_len}s</td></tr>\n",$s3{$k}/$total*100 , $s3{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("Percentage and number of attacks to one host from any with same method\n", "same_dst_attack");
	if ($opt{i}) {
		foreach $k (sort { $s3{$a} <=> $s3{$b} } keys %s3) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${sever_len}s\n",$s3{$k}/$total*100 ,
			$s3{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	} else {
		foreach $k (sort { $s3{$b} <=> $s3{$a} } keys %s3) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${attack_len}s  %-${sever_len}s\n",$s3{$k}/$total*100 ,
			$s3{$k},$_[1],$_[0],$_[2]) if $i < $val_opt_n;
			++$i;
		}
	}
}
}


# to see the popularity of one attack method
sub attack {
if ($opt{h}) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"attack\" href=\"#top\">The distribution of attack methods ( ". keys(%s4) ." of ". keys(%s4) ." )</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Attack</td><td>Priority</td><td>Severity</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s4{$a} <=> $s4{$b} } keys %s4) {
			@_ = split ",",$k;
			if ($link{$_[0]} =~ /(^.+),(.*$)/) {
				$url = init_url($1);
				printf FILEOUT ("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td><a href=$url$2>%-${attack_len}s</a></td><td>%-${prior_len}d</td><td>%-${sever_len}s</td><td>%-20s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]); 
				++$i;
			} else {
				printf FILEOUT ("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td>%-${attack_len}s</td><td>%-${prior_len}d</td><td>%-${sever_len}s</td><td>%-20s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]); 
				++$i;
			}
		}
	} else {
		foreach $k (sort { $s4{$b} <=> $s4{$a} } keys %s4) {
			@_ = split ",",$k;
			if ($link{$_[0]} =~ /(^.+),(.*$)/) {
				$url = init_url($1);
				printf FILEOUT ("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td><a href=$url$2>%-${attack_len}s</a></td><td>%-${prior_len}d</td><td>%-${sever_len}s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]); 
				++$i;
			} else {
				printf FILEOUT ("<tr class=TB><td>%2.2f</td><td>%-${nb_len}d</td><td>%-${attack_len}s</td><td>%-${prior_len}d</td><td>%-${sever_len}s</td></tr>\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]); 
				++$i;
			}
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("The distribution of attack methods\n", "attack");
	if ($opt{i}) {
		foreach $k (sort { $s4{$a} <=> $s4{$b} } keys %s4) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${attack_len}s  %-${prior_len}d %-${sever_len}s\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]); 
			++$i;
		}
	} else {
		foreach $k (sort { $s4{$b} <=> $s4{$a} } keys %s4) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${attack_len}s  %-${prior_len}d %-${sever_len}s\n", $s4{$k}/$total*100,$s4{$k},$_[0],$_[1],$_[2]); 
			++$i;
		}
	}
}
}


# to see the popularity of one source host 
sub same_class {
if ($opt{h}) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"same_class\" href=\"#top\">The distribution of classification method</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Classification</td><td>Severity</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s10{$a} <=> $s10{$b} } keys %s10) {
			@_ = split ",",$k;
			$STYLE = init_color($_[1]);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${class_len}s</td><td>%-${server_len}s</td></tr>\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	} else {
		foreach $k (sort { $s10{$b} <=> $s10{$a} } keys %s10) {
			@_ = split ",",$k;
			$STYLE = init_color($_[1]);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${class_len}s</td><td>%-${server_len}s</td></tr>\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("The distribution of classification method\n", "same_class");
	if ($opt{i}) {
		foreach $k (sort { $s10{$a} <=> $s10{$b} } keys %s10) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${class_len}s  %-${server_len}s\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	} else {
		foreach $k (sort { $s10{$b} <=> $s10{$a} } keys %s10) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${class_len}s  %-${sever_len}s\n", $s10{$k}/$total*100,$s10{$k},$_[0],$_[1]); 
		}
	}
}
}



# to see the popularity of one source host 
sub attack_src {
$i=0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s5) ) ) {
	$val_opt_n = keys %s5;
}
if ( $opt_h == 1 ) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"attack_src\" href=\"#top\">To see the popularity of one source host ( ". $val_opt_n ." of ". keys(%s5) ." )</a><CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	if ($opt{r}) {
		print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td><td>Resolve</td><td>Domain</td></tr>");
	} else {
		print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td></tr>");
	}
	
	if ($opt{i}) {
		foreach $k (sort { $s5{$a} <=> $s5{$b} } keys %s5) {
			if ($i < $val_opt_n) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$STYLE = init_color($src_host);
					$src_host =~ /(\w+$)/;
					printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${addr_len}s</td><td>%-${domain_len}s</td></tr>\n",$s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s5{$k}/$total*100,$s5{$k},$k);
				}
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
			if ($i < $val_opt_n) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$STYLE = init_color($src_host);
					$src_host =~ /(\w+$)/;
					printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${resolve_len}s</td><td>%-${domain_len}s</td></tr>\n",$s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					 printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s5{$k}/$total*100,$s5{$k},$k);
				}
			}
		++$i;
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	if ($opt{r}) {
		section_header("To see the popularity of one source host\n", "attack_src_resolve");
	} else {
		section_header("To see the popularity of one source host\n", "attack_src");
	}
	if ($opt{i}) {
		foreach $k (sort { $s5{$a} <=> $s5{$b} } keys %s5) {
			if ($i < $val_opt_n) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$src_host =~ /(\w+$)/;
					$s40{"$1"}++;
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s  %-${domain_len}s\n", $s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s5{$k}/$total*100,$s5{$k},$k) if $i < $nb; 
				}
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
			if ($i < $val_opt_n) {
				if ($opt{r}) {
					$host = resolve ($k);
					$src_host = $host if ($host ne $k);
					$src_host =~ /(\w+$)/;
#					$s40{"$1"}++;
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s  %-${domain_len}s\n", $s5{$k}/$total*100,$s5{$k},$k,$src_host,$DomainName{$1});
				} else {
					printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s5{$k}/$total*100,$s5{$k},$k);
				}
			}
		++$i;
		}
	}

}
}




sub domain_src {
if ( $opt_h == 1 ) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"domain_src\" HREF=\"#top\">To see the popularity of one domain ( Top : ". $nb ." )</a><CENTER></h3>\n");
	print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Domain</td></tr>");
	foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
		$host = resolve ($k);
		$src_host = $host if ($host ne $k);
		$src_host =~ /(\w+$)/;
		$s40{"$1"}++;
	}
	if ($opt{i}) {
		foreach $k (sort { $s40{$a} <=> $s40{$b} } keys %s40) {
			$STYLE = init_color($k);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${domain_len}s</td></tr>\n", $s40{$k}/(keys %s5)*100,$s40{$k},$DomainName{$k});
			push @list1 ,($s40{$k});
			push @list2 ,("$k");
			++$i;
		}
	} else {
		foreach $k (sort { $s40{$b} <=> $s40{$a} } keys %s40) {
			$STYLE = init_color($k);
			printf FILEOUT ("<tr class=$STYLE><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${domain_len}s</td></tr>\n", $s40{$k}/(keys %s5)*100,$s40{$k},$DomainName{$k});
			push @list1 ,($s40{$k});
			push @list2 ,("$k");
			++$i;
		}
	}
	if ( $opt{g} ) {
		init_graph("domain","pie") if $opt{g};
		print FILEOUT ("</TABLE></TD><TD WIDTH=300 ALIGN=right><IMG SRC=".$val_opt_h._domain_.$dategraph.".".$opt{g}."></TD></TR></TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE></TD></TR></TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	foreach $k (sort { $s5{$b} <=> $s5{$a} } keys %s5) {
		$host = resolve ($k);
		$src_host = $host if ($host ne $k);
		$src_host =~ /(\w+$)/;
		$s40{"$1"}++;
	}
	section_header("To see the popularity of one domain\n", "domain_src");
	if ($opt{i}) {
		foreach $l (sort { $s40{$a} <=> $s40{$b} } keys %s40) {
			printf("  %5.2f  %-${nb_len}d  %-${domain_len}s\n", $s40{$l}/(keys %s5)*100,$s40{$l},$DomainName{$l});
		}
	} else {
		foreach $l (sort { $s40{$b} <=> $s40{$a} } keys %s40) {
			printf("  %5.2f  %-${nb_len}d  %-${domain_len}s\n", $s40{$l}/(keys %s5)*100,$s40{$l},$DomainName{$l});
		}
	}
}
}



# to see the popularity of one destination host
sub attack_dst {
$i=0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s6) ) ) {
	$val_opt_n = keys %s6;
}
if ( $opt_h == 1 ) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"attack_dst\" href=\"#top\">To see the popularity of one destination host ( ". $val_opt_n ." of ". keys(%s6) ." )</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	if ($opt{r}) {
		print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Destination</td><td>Resolve</td></tr>");
	} else {
		print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Destination</td></tr>");
	}

	if ($opt{i}) {
		foreach $k (sort { $s6{$a} <=> $s6{$b} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${resolve_len}s</td></tr>\n",$s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $val_opt_n;
			} else {
				printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $val_opt_n;
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s6{$b} <=> $s6{$a} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td><td>%-${resolve_len}s</td></tr>\n",$s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $val_opt_n;
			} else {
				printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${addr_len}s</td></tr>\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $val_opt_n;
			}
		++$i;
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	if ($opt{r}) {
		section_header("To see the popularity of one destination host\n", "attack_dst_resolve");
	} else {
		section_header("To see the popularity of one destination host\n", "attack_dst");
	}
	if ($opt{i}) {
		foreach $k (sort { $s6{$a} <=> $s6{$b} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s\n", $s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $val_opt_n;
			} else {
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $val_opt_n; 
			}
		++$i;
		}
	} else {
		foreach $k (sort { $s6{$b} <=> $s6{$a} } keys %s6) {
			if ($opt{r}) {
				$host = resolve ($k);
				$src_host = $host if ($host ne $k);
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s  %-${resolve_len}s\n", $s6{$k}/$total*100,$s6{$k},$k,$src_host) if $i < $val_opt_n;
			} else {
				printf("  %5.2f  %-${nb_len}d  %-${addr_len}s\n", $s6{$k}/$total*100,$s6{$k},$k) if $i < $val_opt_n; 
			}
		++$i;
		}
	}

}
}



# to see the activity by hour 
sub hour {
$max = 0;
if ( $opt_h == 1 ) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<H3><CENTER><A NAME=\"hour\" HREF=\"#top\">The distribution of attack by hour</A><CENTER></H3>\n");
	print FILEOUT ("<TABLE BORDER=5 ALIGN=center>\n");
	print FILEOUT ("<TR CLASS=TH ALIGN=center><TD>Hour</TD><TD>Nb</TD><TD>%</TD><TD>Graph</TD></TR>");
	

	foreach $k (%s7) {
		$max = $s7{$k} if ($s7{$k} > $max);
	}
	 my $ratio = $max/$total*100;
	foreach $k (sort { $a <=> $b } keys %s7) {
		my $num = $s7{$k}/$total*(50000/$ratio);
		printf FILEOUT ("<TR CLASS=TB><TD>%${hour_len}dh</TD><TD>%-${nb_len}d</TD><TD>%-2.2f</TD><TD><HR NOSHADE ALIGN=left SIZE=10 COLOR=#CCCCCC  WIDTH=%3d ></TD></TR>\n", $k,$s7{$k},$s7{$k}/$total*100,$num); 
		push @list1 ,($s7{$k});
		push @list2 ,($k);
	}
	init_graph("hours1","area") if $opt{g};
	print FILEOUT ("</TD></TR></TABLE><BR>\n");
	$compt = 1;	
	foreach $k (sort { $a <=> $b } keys %s8) {
		@_ = split ",",$k;
		$beforeday = $_[0];
		$belowday = $beforeday if ( $compt == 1 );
		if ( $belowday < $beforeday ) {
			if ( $low ) {
				push @listlow ,($low);
				undef $low;
			} else {
				push @listlow ,(0);
			}
			if ( $medium ) {
				push @listmedium ,($medium);
				undef $medium;
			} else {
				push @listmedium ,(0);
			}
			if ( $high ) {
				push @listhigh ,($high);
				undef $high;
			} else {
				push @listhigh ,(0);
			}
			if ( $unknown ) {
				push @listunknown ,($unknown);
				undef $unknown;
			} else {
				push @listunknown ,(0);
			}
		}
		if ( $_[1] eq "low") {
			$low = $s8{$k};
		} elsif ( $_[1] eq "medium" ) {
			$medium = $s8{$k};
		} elsif ( $_[1] eq "high" ) {
			$high = $s8{$k};
		} elsif ( $_[1] eq "unknown" ) {
			$unknown = $s8{$k};
		}
		$belowday = $beforeday;
		$compt ++;
	}
	if ( $low ) {
		push @listlow ,($low);
		undef $low;
	} else {
		push @listlow ,(0);
	}
	if ( $medium ) {
		push @listmedium ,($medium);
		undef $medium;
	} else {
		push @listmedium ,(0);
	}
	if ( $high ) {
		push @listhigh ,($high);
	} else {
		push @listhigh ,(0);
	}
	if ( $unknown ) {
		push @listunknown ,($unknown);
		undef $unknown;
	} else {
		push @listunknown ,(0);
	}
	if ( $opt{g} ) {
		init_graph("hours2","bars");
		print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><IMG SRC=".$val_opt_h._hours1_.$dategraph.".".$opt{g}."></TD></TR><BR><TR><TD><IMG SRC=".$val_opt_h._hours2_.$dategraph.".".$opt{g}."></TD></TR></TABLE>\n");
		print FILEOUT ("</TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	foreach $k (%s7) {
		$max = $s7{$k} if ($s7{$k} > $max);
	}
	$ratio = $max/$total*100;
	section_header("The distribution of attack by hour\n", "hour");
	foreach $k (sort keys %s7) {
		my $num = $s7{$k}/$total*75*(100 / $ratio);
		printf("  %${hour_len}dh  %-${nb_len}d  %5.2f  %-${graph_len}s\n", $k,$s7{$k},$s7{$k}/$total*100, '#' x $num); 
	}
}
}


sub same_hour_attack {
$i=0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s9) ) ) {
	$val_opt_n = keys %s9;
}
if ( $opt_h == 1 ) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"hour_attack\" href=\"#top\">Percentage and number of attacks by hour</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Hour</td><td>Attack</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s9{$a} <=> $s9{$b} } keys %s9) {
			@_ = split ",",$k;
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%${hour_len}dh</td><td>%-${attack_len}s</td></tr>\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $val_opt_n;
			++$i
		}
	} else {
		foreach $k (sort { $s9{$b} <=> $s9{$a} } keys %s9) {
			@_ = split ",",$k;
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%${hour_len}dh</td><td>%-${attack_len}s</td></tr>\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $val_opt_n;
			++$i;
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("Percentage and number of attacks by hour\n", "same_hour_attack");  
	if ($opt{i}) {
		foreach $k (sort { $s9{$a} <=> $s9{$b} } keys %s9) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %${hour_len}dh    %-${attack_len}s\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $val_opt_n;
			++$i
		}
	} else {
		foreach $k (sort { $s9{$b} <=> $s9{$a} } keys %s9) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %${hour_len}dh    %-${attack_len}s\n",
			$s9{$k}/$total*100, $s9{$k},$_[0],$_[1]) if $i < $val_opt_n;
			++$i;
		}
	}
}
}


sub dport {
$i = 0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s16) ) ) {
	$val_opt_n = keys %s16;
}
if ( $opt_h == 1 ) {
	@list1 = ();
	@list2 = ();
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"dport\" HREF=\"#top\">Distribution of event by destination port</a></CENTER></h3>\n");
	print FILEOUT ("<TABLE BORDER=0 ALIGN=center><TR><TD><table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Destination Port</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s16{$a} <=> $s16{$b} } keys %s16) {
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${port_len}s</td></tr>\n",
			$s16{$k}/$total*100, $s16{$k},$k) if ( $i < $val_opt_n ) ;
			push @list1 ,($s16{$k});
			push @list2 ,("$k");
			++$i;
		}
	} else {
		foreach $k (sort { $s16{$b} <=> $s16{$a} } keys %s16) {
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${port_len}s</td></tr>\n",
			$s16{$k}/$total*100, $s16{$k},$k) if ( $i < $val_opt_n );
			push @list1 ,($s16{$k});
			push @list2 ,("$k");
			++$i;
		}
	}
	if ( $opt{g} ) {
		init_graph("dport","pie");
		print FILEOUT ("</TABLE></TD><TD WIDTH=300 ALIGN=right><IMG SRC=".$val_opt_h._dport_.$dategraph.".".$opt{g}."></TD></TR></TABLE><BR><HR>\n");
	} else {
		print FILEOUT ("</TABLE></TD></TR></TABLE><BR><HR>\n");
	}
	close (FILEOUT);
} else {
	section_header("Distribution of event by destination port\n", "dport");  
	if ($opt{i}) {
		foreach $k (sort { $s16{$a} <=> $s16{$b} } keys %s16) {
			printf("  %5.2f  %-${nb_len}d  %-${port_len}s\n",
			$s16{$k}/$total*100, $s16{$k},$k) if ( $i < $val_opt_n );
			++$i;
		}
	} else {
		foreach $k (sort { $s16{$b} <=> $s16{$a} } keys %s16) {
			printf("  %5.2f  %-${nb_len}d  %-${port_len}s  %-${attack_len}s\n",
			$s16{$k}/$total*100, $s16{$k},$k) if ( $i < $val_opt_n );
			++$i;
		}
	}
}
}




sub dport_attack {
$i = 0;
if ( ( $opt_n != 1 ) || ( $val_opt_n > keys(%s18) ) ) {
	$val_opt_n = keys %s18;
}
if ( $opt_h == 1 ) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"port_attack\" href=\"#top\">Percentage and number of attacks to one destination port grouped by attack</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>Port</td><td>Attack</td></tr>");
	if ($opt{i}) {
		foreach $k (sort { $s18{$a} <=> $s18{$b} } keys %s18) {
			@_ = split ",",$k;
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${port_len}s</td><td>%-${attack_len}s</td></tr>\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $val_opt_n );
			++$i;
		}
	} else {
		foreach $k (sort { $s18{$b} <=> $s18{$a} } keys %s18) {
			@_ = split ",",$k;
			printf FILEOUT ("<tr class=TB><td>%-2.2f</td><td>%-${nb_len}d</td><td>%-${port_len}s</td><td>%-${attack_len}s</td></tr>\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $val_opt_n );
			++$i;
		}
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("Percentage and number of attacks to one destination port grouped by attack\n", "dport_attack");  
	if ($opt{i}) {
		foreach $k (sort { $s18{$a} <=> $s18{$b} } keys %s18) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${port_len}s  %-${attack_len}s\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $val_opt_n );
			++$i;
		}
	} else {
		foreach $k (sort { $s18{$b} <=> $s18{$a} } keys %s18) {
			@_ = split ",",$k;
			printf("  %5.2f  %-${nb_len}d  %-${port_len}s  %-${attack_len}s\n",
			$s18{$k}/$total*100, $s18{$k},$_[0],$_[1]) if ( $i < $val_opt_n );
			++$i;
		}
	}
}
}


sub init_domains {
open(DOMAINS,$domains_file) or die "Can not open domain file: $domains_file $!\n";
$MaxDomain=0;
while (<DOMAINS>) {
	next if (/^$/ or /^#/);
	$Dots=1;
	($Code,$Description) = /(^\S+)\s+(.*)/;
	$Code =~ tr/A-Z/a-z/;
	$DomainName{$Code}="$Description";
	$Dots++ while $Code =~ /\./g;
	$MaxDomain=$Dots if ($Dots >$MaxDomain);
}
close(DOMAINS);
}



sub init_rules {
open(RULES,$rules_file) or die "Can not open domain file: $rules_file $!\n";
while (<RULES>) {
next if (/^$/ or /^#/);
($descr_attack,$link_attack) = /(^.+})\s+(.*)$/;
$link{$descr_attack}="$link_attack";
}
close(RULES);
}



sub init_color {
my $LOG = shift;
my $COLOR;
if (($LOG eq "high") || ($LOG =~ /nresolved/)) {
	$COLOR="TBR";
} elsif ($LOG eq "unknown") {
	$COLOR="TBG";
} else {
	$COLOR="TB";
}
return ($COLOR)
}




sub init_url {
my $LOG = shift;
my $URL;
if ($LOG eq "bugtraq") {
	$URL="http://www.securityfocus.com/bid/";
} elsif ($LOG eq "arachnids") {
	$URL="http://www.whitehats.com/info/IDS";
} elsif ($LOG eq "cve") {
	$URL="http://cve.mitre.org/cgi-bin/cvename.cgi?name=";
} elsif ($LOG eq "nessus") {
	$URL="http://cgi.nessus.org/plugins/dump.php3?id=";
} elsif ($LOG eq "MCAFEE") {
	$URL="http://vil.nai.com/vil/content/v_";
} elsif ($LOG eq "url") {
	$URL="http://";
}
return ($URL);
}

# portscan (if enable -p switch)
# Please use '-A fast' to generate the log, so portscan() can process it.
# contributed by: Paul Bobby, <paul.bobby@lmco.com>
#                 Jian-Da Li, <jdli@freebsd.csie.nctu.edu.tw>
sub portscan {
my (%s1, %s2);
# to see how many times a host performs portscan
# used in portscan()
for $i (0 .. $#portscan) {
	$s1{"$portscan[$i]->[0]"}++;
}

if ( $opt_h == 1 ) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<h3><CENTER><a name=\"portscan\" href=\"#top\">Portscan List</a></CENTER></h3>\n");
	print FILEOUT ("<table border=5 ALIGN=center>\n");
	print FILEOUT ("<tr class=TH align=center><td>%</td><td>Nb</td><td>IP Source</td></tr>");
	foreach $k (sort { $s1{$b} <=> $s1{$a} } keys %s1) {
		printf FILEOUT ("<tr class=TB><TD>%-2.2f</TD><TD>%-${nb_len}d</TD><TD>%-${addr_len}s\n", $s1{$k}/$totalscan*100,$s1{$k},$k);
	}
	print FILEOUT ("</table><BR><hr>\n");
	close (FILEOUT);
} else {
	section_header("Portscans performed to/from HOME_NET\n", "portscan");
	foreach $k (sort { $s1{$b} <=> $s1{$a} } keys %s1) {
		printf("  %5.2f  %-4d   %-${addr_len}s\n", $s1{$k}/$totalscan*100,$s1{$k},$k);
	}
}
}

# anomsensor (if enable -n switch)
# This function process data generated by spp_anomsensor plug-in (SPADE)
# By Yen-Ming Chen <chenym@alumni.cmu.edu>
sub anomsensor {
my (%s7);
# to see how many times a host performs portscan
# used in anomsensor()
for $i (0 .. $#anores) {
$s7{"$anores[$i]->[1],$anores[$i]->[3],$anores[$i]->[4]"}++;
}
section_header("Anomaly detected by SPADE\n", "asdo");
foreach $k (sort { $s7{$b} <=> $s7{$a} } keys %s7) {
@_ = split(/,/,$k);
printf("   %-4d   %-${addr_len}s %-${addr_len}s\t%-6d\n", $s7{$k},$_[0],$_[1],$_[2]) if $s7{$k};
}
}

# print the footer (needed for html)
sub print_footer {
if ( $opt_h == 1 ) {
	open (FILEOUT, ">>$val_opt_h") or die "Can no access file : $val_opt_h\n";
	print FILEOUT ("<CENTER>Generated by <a href=http://jeremy.chartier.free.fr/snortalog/>Snortalog</a> Version: $version (Date: $datever )</CENTER>\n");
	print FILEOUT ("</BODY>\n</HTML>");
	close (FILEOUT);
} else {
print <<FootMessage

Version: $version
Jrmy CHARTIER, <jeremy.chartier\@free.fr>
Date: $datever
FootMessage
}
}


#
# resolve host name and cache it
#
sub resolve {

my $Address=shift;
my $Hostname;

if ($Address =~ /(\d+\.\d+\.\d+\.\d+)/) {
$Hostname = gethostbyaddr(inet_aton($Address),AF_INET) or $Hostname="unresolved";
}
return $Hostname;
}








# Use a title and a short code to write the section headers
# This is used in place of a FORMAT as this allows variable column widths
# contributed by: Ned Patterson, <jpatter@alum.mit.edu>
#
sub section_header {
my $linelength;
$title = shift; 
$report = shift;
$_ = shift;
print("\n\n$title");
# constant for method length for now
#  $linelenght = (/p/?7:0) + (/a/?20:0) + (/s/?$addr_len:0) +
#(/d/?$addr_len+3:0) + (/m/?20:0);
#$linelength = 36;
#  print( '=' x $linelength, "\n");

if ( $report eq "attack_src_resolve") {
$linelength = 11 + $nb_len + 2 + $addr_len + 2 + $resolve_len + 2 + $domain_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s5) ." ###\n");
print("    %    nb      IP source        Resolve                                             Domain\n");
} elsif ($report eq "domain_src") {
$linelength = 11 + $domain_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s40) ." ###\n");
print("    %    nb      Domain\n");
} elsif ($report eq "attack_dst_resolve") {
$linelength = 11 + $nb_len + 2 + $addr_len + 2 + $resolve_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s6) ." ###\n");
print("    %    nb      IP destination   Resolve\n");
} elsif ($report eq "attack") {
$linelength = 13 + $nb_len + 4 + $attack_len + 4 + $prior_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s4) ." of ". keys(%s4) ." ###\n");
print("    %    nb      Attack                                                                               Priority Severity\n");
} elsif ($report eq "stateful") {
$linelength = 40;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $anomaly ." of ". $anomaly ." ###\n");
print("    %    nb      Preprocessor\n");
} elsif ($report eq "nids") {
$linelength = 50;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s13) ." of ". keys(%s13) ." ###\n");
print("    %    nb      Sondes\n");
} elsif ($report eq "proto") {
$linelength = 50;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s11) ." of ". keys(%s11) ." ###\n");
print("    %    nb      Protocoles\n");
} elsif ($report eq "interfaces") {
$linelength = 40;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s1) ." of ". keys(%s1) ." ###\n");
print("    %    nb      Interfaces\n");



#} elsif ($report eq "port") {
#$linelength = 30;
#print( '=' x $linelength, "\n");
#if ($opt{n}) {
#$nb = $opt{n};
#} else {
#$nb = keys(%s8);
#}
#print(" " x 4, "### ". keys(%s8) ." of ". keys(%s8) ." ###\n");
#print("    %      nb        Port\n");
} elsif ($report eq "same_class") {
$linelength = 13 + $nb_len + 4 + $class_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s10) ." of ". keys(%s10) ." ###\n");
print("    %    nb       Classification                                               Severity\n");
} elsif ($report eq "same_hour_attack") {
$linelength = 11 + $nb_len + 4 + $hour_len + 4 + $attack_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s9) ." ###\n");
print("    %    nb      Heure  Attack\n");
} elsif ($report eq "same_src_attack") {
$linelength = 11 + $nb_len + 4 + $addr_len + 4 + $attack_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s2) ." ###\n");
print("    %    nb      IP source        Attack                                                                                     Severity\n");
} elsif ($report eq "same_dst_attack") {
$linelength = 13 + $nb_len + 4 + $addr_len + 4 + $attack_len + 4 + $sever_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s3) ." ###\n");
print("    %    nb      IP destination   Attack                                                                                      Severity\n");
} elsif ($report eq "same_src_dst_attack") {
$linelength = 13 + $nb_len + 4 + $addr_len + 4 + $addr_len + 4 + $attack_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s0) ." ###\n");
print("    %    nb      IP source        IP destination   Attack\n");
} elsif ($report eq "dport_attack") {
$linelength = 11 + $nb_len + 4 + $port_len + 4 +  $attack_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s18) ." ###\n");
print("    %    nb      Port   Attack\n");
} elsif ($report eq "dport") {
$linelength = 11 + $nb_len + 4 + $port_len + 4;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s16) ." ###\n");
print("    %    nb      Port\n");
} elsif ($report eq "daily_event") {
$linelength = 20 + $nb_len + 6 + $graph_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s19) ." of ". keys(%s19) ." ###\n");
print(" Day Month  nb         %    Graph\n");
} elsif ($report eq "hour") {
$linelength = 13 + $nb_len + 2 + $hour_len + 2 + $graph_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s7) ." of ". keys(%s7) ." ###\n");
print("  Hour nb        %    Graph\n");
} elsif ($report eq "portscan") {
$linelength = 40;
print( '=' x $linelength, "\n");
print(" " x 4, "### Portscan List ###\n");
print("    %    nb      IP destination\n");
} elsif ($report eq "attack_dst") {
$linelength = 13 + $nb_len + 4 + $addr_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s6) ." ###\n");
print("    %    nb      IP destination\n");
} elsif ($report eq "attack_src") {
$linelength = 13 + $nb_len + 4 + $addr_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". $val_opt_n ." of ". keys(%s5) ." ###\n");
print("    %    nb      IP source\n");
} elsif ($report eq "severity") {
$linelength = 13 + $nb_len + 2 + $sever_len + 2 + $graph_len;
print( '=' x $linelength, "\n");
print(" " x 4, "### ". keys(%s20) ." of ". keys(%s20) ." ###\n");
print("    %    nb      Severity Graph\n");
}
print( '=' x $linelength, "\n");
}



# Put data $alert into matrix for further process
# INPUT: $alert
sub process_data() {
$self = shift;
# put those data into a big matrix
if ($self->{PLUGIN} eq "anomsensor") {
push @anores , [$self->{THR},$self->{SADDR},$self->{SPORT},
	    $self->{DADDR},$self->{DPORT}];
#$opt_n = 1;
} elsif ($self->{PLUGIN} eq "portscan") {
	push @portscan , [$self->{SADDR}];
#$opt_p = 1;
} elsif ($self->{TYPE} eq "sys" || $self->{TYPE} eq "alert" || 
	$self->{PLUGIN} eq "stream4" || $self->{PLUGIN} eq "bo" ||
	$self->{PLUGIN} eq "frag2" || $self->{PLUGIN} eq "snort_decoder" ) {
	$self->{SIG} =~ s/\:$//o;
	push @result , [$self->{MON},$self->{DAY},$self->{HOUR},$self->{MIN},
                   $self->{SEC},$self->{HOST},$self->{SIG},$self->{SADDR},
                   $self->{SPORT},$self->{DADDR},$self->{DPORT},$self->{CLASS},
		   $self->{PLUGIN},$self->{PRIORITY},$self->{SEVERITY},$self->{ETHER},
		   $self->{PROTO}];
    $lastwassnort = 1;
  } else {
    print STDERR "Unknown alert type plugin! $self->{TYPE}:$self->{PLUGIN} Skipped!\n" if $opt{d};
    return;
  }
#  1;
}


sub usage {
print <<Helpmsg
Usage: cat <alerts file> or <snort.rules> | $0 <stats options> <options>

Options:
-i 			Inverse the result	
-r 			Resolve IP adresses	
-c 			Resolve domains	
-d 			Mode debug	
-h 			For specify a HTML file
-n <integer>		For specify a number of line in the result
-g <gif|png|jpg>	For choosing an output format for your graphs (depending of your perl's librairies)
-ether <interface>	For specify an interface
-genref			Generate the reference rules file	
-help			View this help

Reports:
-src                      Top IPs sources
-dst                      Top IPs destination
-src_attack               Top IPs sources grouped by attack
-dst_attack               Top IPs destination grouped by attack
-src_dst_attack           Top alert grouped by IPs sources, Ips destination and attack
-attack                   Top attack
-class                    Top classification
-severity                 Top severity
-daily_event              Top number of attack grouped by day
-hour                     Top number of attack grouped by hour
-hour_attack              Top specific attack grouped by hour
-dport                    Top destination port
-proto                    Top protocoles
-dport_attack             Top destination port grouped by attack
-nids                     Top NIDS host
-stateful                 Top stateful problems
-interfaces               Top interfaces events 
-domain_src               Top of domain source
-portscan                 Top of portscan alert

print_footer();
Helpmsg
;

	exit 0;
}

